import dataclasses
import datetime
import typing

from types_aiobotocore_s3 import S3Client
from types_aiobotocore_s3.type_defs import GetObjectOutputTypeDef

from safe_s3_storage.exceptions import FailedToReplaceS3BaseUrlWithProxyBaseUrlError, InvalidS3PathError
from safe_s3_storage.file_validator import ValidatedFile


_REQUIRED_S3_PATH_PARTS_COUNT: typing.Final = 2


def _extract_bucket_name_and_object_key(s3_path: str) -> tuple[str, str]:
    path_parts: typing.Final = tuple(s3_path.strip("/").split("/", 1))
    if len(path_parts) != _REQUIRED_S3_PATH_PARTS_COUNT:
        raise InvalidS3PathError(s3_path=s3_path)
    return path_parts


@dataclasses.dataclass(kw_only=True, slots=True, frozen=True)
class UploadedFile(ValidatedFile):
    s3_path: str


@dataclasses.dataclass(kw_only=True, slots=True, frozen=True)
class S3Service:
    s3_client: S3Client

    async def upload_file(
        self,
        validated_file: ValidatedFile,
        *,
        bucket_name: str,
        object_key: str,
        metadata: dict[str, str] | None = None,
    ) -> UploadedFile:
        await self.s3_client.put_object(
            Body=validated_file.file_content,
            Bucket=bucket_name,
            Key=object_key,
            ContentType=validated_file.mime_type,
            Metadata=metadata or {},
        )
        return UploadedFile(
            file_name=validated_file.file_name,
            file_content=validated_file.file_content,
            file_size=validated_file.file_size,
            mime_type=validated_file.mime_type,
            s3_path=f"{bucket_name}/{object_key}",
        )

    async def _retrieve_file_object(self, *, s3_path: str) -> GetObjectOutputTypeDef:
        bucket_name, object_key = _extract_bucket_name_and_object_key(s3_path)
        return await self.s3_client.get_object(Bucket=bucket_name, Key=object_key)

    async def stream_file(self, *, s3_path: str, read_chunk_size: int = 70 * 1024) -> typing.AsyncIterator[bytes]:
        file_object: typing.Final = await self._retrieve_file_object(s3_path=s3_path)
        object_body: typing.Final = file_object["Body"]
        while one_chunk := await object_body.read(read_chunk_size):
            yield one_chunk

    async def read_file(self, *, s3_path: str) -> bytes:
        file_object: typing.Final = await self._retrieve_file_object(s3_path=s3_path)
        return await file_object["Body"].read()

    async def create_file_url(
        self, *, s3_path: str, display_file_name: str, expires_in: datetime.timedelta, proxy_base_url: str | None = None
    ) -> str:
        bucket_name, object_key = _extract_bucket_name_and_object_key(s3_path)
        expires_in_seconds: typing.Final = round(expires_in.total_seconds())
        presigned_url: typing.Final = await self.s3_client.generate_presigned_url(
            "get_object",
            Params={
                "Bucket": bucket_name,
                "Key": object_key,
                "ResponseContentDisposition": f'inline; filename="{display_file_name}"',
                "ResponseCacheControl": f"max-age={expires_in_seconds}, public",
            },
            ExpiresIn=expires_in_seconds,
        )

        if proxy_base_url is None:
            return presigned_url
        if presigned_url == (
            presigned_url_without_prefix := presigned_url.removeprefix(
                self.s3_client.meta.endpoint_url.removesuffix("/")
            )
        ):
            raise FailedToReplaceS3BaseUrlWithProxyBaseUrlError(
                s3_file_presigned_url=presigned_url, proxy_base_url=proxy_base_url
            )
        return proxy_base_url.removesuffix("/") + presigned_url_without_prefix



import typing

import faker

from safe_s3_storage.exceptions import KasperskyScanEngineThreatDetectedError
from tests.conftest import generate_binary_content


def test_exception_str(faker: faker.Faker) -> None:
    response: typing.Final = generate_binary_content(faker)
    file_name: typing.Final = faker.file_name()
    assert (
        str(KasperskyScanEngineThreatDetectedError(response=response, file_name=file_name))
        == f"({response=}, {file_name=})"
    )



import typing

import faker
import pytest
import stamina


@pytest.fixture(scope="session", autouse=True)
def anyio_backend() -> str:
    return "asyncio"


@pytest.fixture(scope="session", autouse=True)
def deactivate_retries() -> None:
    stamina.set_active(False)


MIME_OCTET_STREAM: typing.Final = "application/octet-stream"


def generate_binary_content(faker: faker.Faker) -> bytes:
    return faker.binary(length=faker.pyint(min_value=10, max_value=100))



import base64
import dataclasses
import enum
import typing

import httpx
import pydantic
import stamina

from safe_s3_storage.exceptions import KasperskyScanEngineThreatDetectedError


class KasperskyScanEngineRequest(pydantic.BaseModel):
    timeout: str
    object: str
    name: str


# https://support.kaspersky.ru/scan-engine/2.1/193001
class KasperskyScanEngineScanResult(str, enum.Enum):
    CLEAN = "CLEAN"
    DETECT = "DETECT"
    DISINFECTED = "DISINFECTED"
    DELETED = "DELETED"
    NON_SCANNED = "NON_SCANNED"
    SERVER_ERROR = "SERVER_ERROR"


class KasperskyScanEngineResponse(pydantic.BaseModel):
    scanResult: KasperskyScanEngineScanResult  # noqa: N815


@dataclasses.dataclass(kw_only=True, slots=True, frozen=True)
class KasperskyScanEngineClient:
    httpx_client: httpx.AsyncClient
    service_url: str
    client_name: str
    timeout_ms: int = 10000
    max_retries: int = 3

    async def _send_scan_memory_request(self, payload: dict[str, typing.Any]) -> bytes:
        response: typing.Final = await self.httpx_client.post(url=self.service_url, json=payload)
        response.raise_for_status()
        return response.content

    async def scan_memory(self, *, file_name: str, file_content: bytes) -> None:
        payload: typing.Final = KasperskyScanEngineRequest(
            timeout=str(self.timeout_ms), object=base64.b64encode(file_content).decode(), name=self.client_name
        ).model_dump(mode="json")
        response: typing.Final = await stamina.retry(on=httpx.HTTPError, attempts=self.max_retries)(
            self._send_scan_memory_request
        )(payload)
        validated_response: typing.Final = KasperskyScanEngineResponse.model_validate_json(response)
        if validated_response.scanResult == KasperskyScanEngineScanResult.DETECT:
            raise KasperskyScanEngineThreatDetectedError(response=response, file_name=file_name)



import dataclasses
import enum
import typing

import puremagic
import pyvips  # type: ignore[import-untyped]

from safe_s3_storage import exceptions
from safe_s3_storage.kaspersky_scan_engine import KasperskyScanEngineClient


@dataclasses.dataclass(kw_only=True, slots=True, frozen=True)
class ValidatedFile:
    file_name: str
    file_content: bytes
    file_size: int
    mime_type: str


def _is_image(mime_type: str) -> bool:
    return mime_type.startswith("image/")


class ImageConversionFormat(str, enum.Enum):
    jpeg = enum.auto()
    webp = enum.auto()


_IMAGE_CONVERSION_FORMAT_TO_MIME_TYPE_AND_EXTENSION_MAP: typing.Final = {
    ImageConversionFormat.jpeg: ("image/jpeg", "jpg"),
    ImageConversionFormat.webp: ("image/webp", "webp"),
}


def _split_file_base_name_and_extensions(file_name: str) -> tuple[str, str | None]:
    split_result: typing.Final = file_name.rsplit(".", 1) or [file_name]
    return split_result[0], None if len(split_result) == 1 else split_result[1]


@dataclasses.dataclass(kw_only=True, slots=True, frozen=True)
class FileValidator:
    kaspersky_scan_engine: KasperskyScanEngineClient | None = None
    allowed_mime_types: list[str]
    scan_images_with_antivirus: bool = True
    max_file_size_bytes: int = 10 * 1024 * 1024  # 10 MB
    max_image_size_bytes: int = 50 * 1024 * 1024  # 50 MB
    image_conversion_format: ImageConversionFormat = ImageConversionFormat.webp
    image_quality: int = 85

    def _validate_mime_type(self, *, file_name: str, file_content: bytes) -> str:
        try:
            mime_type = puremagic.from_string(file_content, mime=True)
        except puremagic.PureError:
            # unlike python-magic, puremagic doesn't recognize if text is binary
            try:
                file_content.decode()
            except UnicodeDecodeError:
                mime_type = "application/octet-stream"
            else:
                mime_type = "text/plain"
        if mime_type in self.allowed_mime_types:
            return mime_type
        raise exceptions.NotAllowedMimeTypeError(
            file_name=file_name, mime_type=mime_type, allowed_mime_types=self.allowed_mime_types
        )

    def _validate_file_size(self, *, file_name: str, file_content: bytes, mime_type: str) -> int:
        content_size: typing.Final = len(file_content)
        max_size: typing.Final = self.max_image_size_bytes if _is_image(mime_type) else self.max_file_size_bytes
        if content_size > max_size:
            raise exceptions.TooLargeFileError(
                file_name=file_name, file_size=content_size, mime_type=mime_type, max_size=max_size
            )
        return content_size

    def _convert_image(self, validated_file: ValidatedFile) -> ValidatedFile:
        if not _is_image(validated_file.mime_type):
            return validated_file

        target_mime_type, target_extension = _IMAGE_CONVERSION_FORMAT_TO_MIME_TYPE_AND_EXTENSION_MAP[
            self.image_conversion_format
        ]

        try:
            pyvips_image: typing.Final[pyvips.Image] = pyvips.Image.new_from_buffer(
                validated_file.file_content, options=""
            )
            new_file_content: typing.Final = typing.cast(
                "bytes",
                pyvips_image.write_to_buffer(f".{target_extension}", Q=self.image_quality),
            )
        except pyvips.Error as pyvips_error:
            raise exceptions.FailedToConvertImageError(
                file_name=validated_file.file_name, mime_type=validated_file.mime_type
            ) from pyvips_error

        file_base_name, _file_extension = _split_file_base_name_and_extensions(validated_file.file_name)
        return ValidatedFile(
            file_name=f"{file_base_name}.{target_extension}",
            file_content=new_file_content,
            file_size=len(new_file_content),
            mime_type=target_mime_type,
        )

    async def validate_file(self, *, file_name: str, file_content: bytes) -> ValidatedFile:
        mime_type: typing.Final = self._validate_mime_type(file_name=file_name, file_content=file_content)
        file_size: typing.Final = self._validate_file_size(
            file_name=file_name, file_content=file_content, mime_type=mime_type
        )
        validated_file: typing.Final = self._convert_image(
            ValidatedFile(file_name=file_name, file_content=file_content, mime_type=mime_type, file_size=file_size)
        )
        if self.kaspersky_scan_engine:
            is_image: typing.Final = _is_image(validated_file.mime_type)
            if (is_image and self.scan_images_with_antivirus) or not is_image:
                await self.kaspersky_scan_engine.scan_memory(
                    file_name=validated_file.file_name, file_content=validated_file.file_content
                )
        return validated_file



import random
import typing

import faker
import httpx
import pytest

from safe_s3_storage import exceptions
from safe_s3_storage.file_validator import (
    _IMAGE_CONVERSION_FORMAT_TO_MIME_TYPE_AND_EXTENSION_MAP,
    FileValidator,
    ImageConversionFormat,
)
from safe_s3_storage.kaspersky_scan_engine import (
    KasperskyScanEngineClient,
    KasperskyScanEngineResponse,
    KasperskyScanEngineScanResult,
)
from tests.conftest import MIME_OCTET_STREAM, generate_binary_content


@pytest.fixture
def png_file() -> bytes:
    return (
        b"\x89PNG\r\n\x1a\n"  # PNG signature
        b"\x00\x00\x00\r"  # IHDR chunk length
        b"IHDR"  # IHDR chunk type
        b"\x00\x00\x00\x01"  # width: 1
        b"\x00\x00\x00\x01"  # height: 1
        b"\x08"  # bit depth: 8
        b"\x06"  # color type: RGBA
        b"\x00"  # compression method
        b"\x00"  # filter method
        b"\x00"  # interlace method
        b"\x1f\x15\xc4\x89"  # CRC for IHDR
        b"\x00\x00\x00\x0a"  # IDAT chunk length
        b"IDAT"  # IDAT chunk type
        b"\x78\x9c\x63\x60\x00\x00\x00\x02\x00\x01"  # compressed image data (deflate)
        b"\x5d\xc6\x2d\xb4"  # CRC for IDAT
        b"\x00\x00\x00\x00"  # IEND chunk length
        b"IEND"  # IEND chunk type
        b"\xae\x42\x60\x82"  # CRC for IEND
    )


def get_mocked_kaspersky_scan_engine_client(*, faker: faker.Faker, ok_response: bool) -> KasperskyScanEngineClient:
    if ok_response:
        all_scan_results: typing.Final[list[KasperskyScanEngineScanResult]] = list(KasperskyScanEngineScanResult)
        all_scan_results.remove(KasperskyScanEngineScanResult.DETECT)
        scan_result = random.choice(all_scan_results)
    else:
        scan_result = KasperskyScanEngineScanResult.DETECT

    scan_response: typing.Final = KasperskyScanEngineResponse(scanResult=scan_result)
    return KasperskyScanEngineClient(
        service_url=faker.url(schemes=["http"]),
        client_name=faker.pystr(),
        httpx_client=httpx.AsyncClient(
            transport=httpx.MockTransport(lambda _: httpx.Response(200, json=scan_response.model_dump(mode="json")))
        ),
    )


class TestFileValidator:
    async def test_fails_to_validate_mime_type(self, faker: faker.Faker) -> None:
        with pytest.raises(exceptions.NotAllowedMimeTypeError):
            await FileValidator(allowed_mime_types=["image/jpeg"]).validate_file(
                file_name=faker.file_name(), file_content=generate_binary_content(faker)
            )

    async def test_fails_to_validate_file_size(self, faker: faker.Faker) -> None:
        with pytest.raises(exceptions.TooLargeFileError):
            await FileValidator(allowed_mime_types=[MIME_OCTET_STREAM], max_file_size_bytes=0).validate_file(
                file_name=faker.file_name(), file_content=generate_binary_content(faker)
            )

    async def test_fails_to_validate_image_size(self, faker: faker.Faker, png_file: bytes) -> None:
        with pytest.raises(exceptions.TooLargeFileError):
            await FileValidator(allowed_mime_types=["image/png"], max_image_size_bytes=0).validate_file(
                file_name=faker.file_name(), file_content=png_file
            )

    async def test_fails_to_convert_image(self, faker: faker.Faker, png_file: bytes) -> None:
        with pytest.raises(exceptions.FailedToConvertImageError):
            await FileValidator(allowed_mime_types=["image/png"]).validate_file(
                file_name=faker.file_name(), file_content=png_file[:50]
            )

    @pytest.mark.parametrize("image_conversion_format", list(ImageConversionFormat))
    async def test_ok_image(
        self, faker: faker.Faker, png_file: bytes, image_conversion_format: ImageConversionFormat
    ) -> None:
        file_base_name: typing.Final = faker.pystr()

        validated_file: typing.Final = await FileValidator(
            allowed_mime_types=["image/png"], image_conversion_format=image_conversion_format
        ).validate_file(file_name=f"{file_base_name}.{faker.file_extension()}", file_content=png_file)

        assert (
            validated_file.file_name
            == f"{file_base_name}.{_IMAGE_CONVERSION_FORMAT_TO_MIME_TYPE_AND_EXTENSION_MAP[image_conversion_format][1]}"
        )
        assert validated_file.file_content != png_file
        assert validated_file.file_size == len(validated_file.file_content)
        assert (
            validated_file.mime_type
            == _IMAGE_CONVERSION_FORMAT_TO_MIME_TYPE_AND_EXTENSION_MAP[image_conversion_format][0]
        )

    @pytest.mark.parametrize("binary", [True, False])
    async def test_ok_not_image(self, faker: faker.Faker, binary: bool) -> None:
        file_name: typing.Final = faker.file_name()
        file_content: typing.Final = generate_binary_content(faker) if binary else faker.pystr().encode()

        validated_file: typing.Final = await FileValidator(
            allowed_mime_types=[MIME_OCTET_STREAM if binary else "text/plain"]
        ).validate_file(file_name=file_name, file_content=file_content)

        assert validated_file.file_name == file_name
        assert validated_file.file_content == file_content
        assert validated_file.file_size == len(file_content)
        assert validated_file.mime_type == MIME_OCTET_STREAM if binary else "text/plain"

    @pytest.mark.parametrize("ok_response", [True, False])
    async def test_antivirus_skips_images(self, faker: faker.Faker, png_file: bytes, ok_response: bool) -> None:
        await FileValidator(
            kaspersky_scan_engine=get_mocked_kaspersky_scan_engine_client(faker=faker, ok_response=ok_response),
            scan_images_with_antivirus=False,
            allowed_mime_types=["image/png"],
        ).validate_file(file_name=faker.file_name(), file_content=png_file)

    async def test_antivirus_fails_on_files(self, faker: faker.Faker) -> None:
        with pytest.raises(exceptions.KasperskyScanEngineThreatDetectedError):
            await FileValidator(
                kaspersky_scan_engine=get_mocked_kaspersky_scan_engine_client(faker=faker, ok_response=False),
                allowed_mime_types=[MIME_OCTET_STREAM],
            ).validate_file(file_name=faker.file_name(), file_content=generate_binary_content(faker))

    async def test_antivirus_fails_on_images(self, faker: faker.Faker, png_file: bytes) -> None:
        with pytest.raises(exceptions.KasperskyScanEngineThreatDetectedError):
            await FileValidator(
                kaspersky_scan_engine=get_mocked_kaspersky_scan_engine_client(faker=faker, ok_response=False),
                allowed_mime_types=["image/png"],
            ).validate_file(file_name=faker.file_name(), file_content=png_file)

    async def test_antivirus_passes_on_files(self, faker: faker.Faker) -> None:
        await FileValidator(
            kaspersky_scan_engine=get_mocked_kaspersky_scan_engine_client(faker=faker, ok_response=True),
            allowed_mime_types=[MIME_OCTET_STREAM],
        ).validate_file(file_name=faker.file_name(), file_content=generate_binary_content(faker))

    async def test_antivirus_passes_on_images(self, faker: faker.Faker, png_file: bytes) -> None:
        await FileValidator(
            kaspersky_scan_engine=get_mocked_kaspersky_scan_engine_client(faker=faker, ok_response=True),
            allowed_mime_types=["image/png"],
        ).validate_file(file_name=faker.file_name(), file_content=png_file)



from safe_s3_storage import exceptions
from safe_s3_storage.file_validator import FileValidator, ImageConversionFormat, ValidatedFile
from safe_s3_storage.kaspersky_scan_engine import KasperskyScanEngineClient
from safe_s3_storage.s3_service import S3Service, UploadedFile


__all__ = [
    "FileValidator",
    "ImageConversionFormat",
    "KasperskyScanEngineClient",
    "S3Service",
    "UploadedFile",
    "ValidatedFile",
    "exceptions",
]



import datetime
import typing
from unittest import mock

import faker
import pytest

from safe_s3_storage.exceptions import FailedToReplaceS3BaseUrlWithProxyBaseUrlError, InvalidS3PathError
from safe_s3_storage.file_validator import FileValidator
from safe_s3_storage.s3_service import S3Service, UploadedFile
from tests.conftest import MIME_OCTET_STREAM, generate_binary_content


class TestS3ServiceUpload:
    async def test_ok_upload(self, faker: faker.Faker) -> None:
        s3_client_mock: typing.Final = mock.AsyncMock()
        file_name: typing.Final = faker.file_name()
        bucket_name: typing.Final = faker.pystr()
        file_content: typing.Final = generate_binary_content(faker)
        file_original_name_key: typing.Final = faker.pystr()

        validated_file: typing.Final = await FileValidator(allowed_mime_types=[MIME_OCTET_STREAM]).validate_file(
            file_name=file_name, file_content=file_content
        )
        uploaded_file: typing.Final = await S3Service(s3_client=s3_client_mock).upload_file(
            validated_file,
            bucket_name=bucket_name,
            object_key=validated_file.file_name,
            metadata={file_original_name_key: file_name},
        )

        assert uploaded_file == UploadedFile(
            file_content=file_content,
            file_name=file_name,
            file_size=len(file_content),
            mime_type=MIME_OCTET_STREAM,
            s3_path=f"{bucket_name}/{file_name}",
        )
        s3_client_mock.put_object.assert_called_once_with(
            Body=file_content,
            Bucket=bucket_name,
            Key=file_name,
            ContentType=MIME_OCTET_STREAM,
            Metadata={file_original_name_key: file_name},
        )


class TestS3ServiceRead:
    async def test_ok_read(self, faker: faker.Faker) -> None:
        file_content: typing.Final = generate_binary_content(faker)
        bucket_name, s3_key = faker.pystr(), faker.pystr()
        s3_client_mock: typing.Final = mock.Mock(
            get_object=mock.AsyncMock(return_value={"Body": mock.Mock(read=mock.AsyncMock(return_value=file_content))})
        )

        read_file: typing.Final = await S3Service(s3_client=s3_client_mock).read_file(s3_path=f"{bucket_name}/{s3_key}")

        s3_client_mock.get_object.assert_called_once_with(Bucket=bucket_name, Key=s3_key)
        assert read_file == file_content

    async def test_ok_stream(self, faker: faker.Faker) -> None:
        file_content_chunks: typing.Final = [
            generate_binary_content(faker) for _ in range(faker.pyint(min_value=2, max_value=10))
        ]
        bucket_name, s3_key = faker.pystr(), faker.pystr()
        s3_client_mock: typing.Final = mock.Mock(
            get_object=mock.AsyncMock(
                return_value={"Body": mock.Mock(read=mock.AsyncMock(side_effect=[*file_content_chunks, ""]))}
            )
        )

        read_chunks: typing.Final = [
            one_chunk
            async for one_chunk in S3Service(s3_client=s3_client_mock).stream_file(s3_path=f"{bucket_name}/{s3_key}")
        ]

        s3_client_mock.get_object.assert_called_once_with(Bucket=bucket_name, Key=s3_key)
        assert read_chunks == file_content_chunks

    async def test_fails_to_parse_s3_path(self, faker: faker.Faker) -> None:
        with pytest.raises(InvalidS3PathError):
            await S3Service(s3_client=mock.Mock()).read_file(s3_path=faker.pystr())


class TestS3ServiceCreateFileUrl:
    async def test_call(self, faker: faker.Faker) -> None:
        bucket_name, s3_key, display_file_name = faker.pystr(), faker.pystr(), faker.pystr()
        presigned_url: typing.Final = faker.url()
        expires_in_seconds: typing.Final = faker.pyint(min_value=1)
        s3_client_mock: typing.Final = mock.Mock(generate_presigned_url=mock.AsyncMock(return_value=presigned_url))

        await S3Service(s3_client=s3_client_mock).create_file_url(
            s3_path=f"{bucket_name}/{s3_key}",
            display_file_name=display_file_name,
            expires_in=datetime.timedelta(seconds=expires_in_seconds),
        )

        assert s3_client_mock.generate_presigned_url.mock_calls == [
            mock.call(
                "get_object",
                Params={
                    "Bucket": bucket_name,
                    "Key": s3_key,
                    "ResponseContentDisposition": f'inline; filename="{display_file_name}"',
                    "ResponseCacheControl": f"max-age={expires_in_seconds}, public",
                },
                ExpiresIn=expires_in_seconds,
            )
        ]

    async def test_returns_ok_without_proxy(self, faker: faker.Faker) -> None:
        presigned_url: typing.Final = faker.url()
        s3_client_mock: typing.Final = mock.Mock(generate_presigned_url=mock.AsyncMock(return_value=presigned_url))

        file_url: typing.Final = await S3Service(s3_client=s3_client_mock).create_file_url(
            s3_path=f"{faker.pystr()}/{faker.pystr()}",
            display_file_name=faker.pystr(),
            expires_in=faker.time_delta(),
        )

        assert file_url == presigned_url

    @pytest.mark.parametrize("endpoint_url", ["http://s3", "http://s3/"])
    @pytest.mark.parametrize("proxy_base_url", ["https://s3-proxy.me.com", "https://s3-proxy.me.com/"])
    async def test_returns_ok_with_proxy(self, faker: faker.Faker, endpoint_url: str, proxy_base_url: str) -> None:
        presigned_url_path: typing.Final = faker.pystr()
        s3_client_mock: typing.Final = mock.Mock(
            generate_presigned_url=mock.AsyncMock(
                return_value=f"{endpoint_url.removesuffix('/')}/{presigned_url_path}"
            ),
            meta=mock.Mock(endpoint_url=endpoint_url),
        )

        file_url: typing.Final = await S3Service(s3_client=s3_client_mock).create_file_url(
            s3_path=f"{faker.pystr()}/{faker.pystr()}",
            display_file_name=faker.pystr(),
            expires_in=faker.time_delta(),
            proxy_base_url=proxy_base_url,
        )

        assert file_url == f"{proxy_base_url.removesuffix('/')}/{presigned_url_path}"

    async def test_fails_with_proxy(self, faker: faker.Faker) -> None:
        presigned_url_path: typing.Final = faker.pystr()
        s3_client_mock: typing.Final = mock.Mock(
            generate_presigned_url=mock.AsyncMock(return_value=f"http://not-that-s3/{presigned_url_path}"),
            meta=mock.Mock(endpoint_url="http://s3"),
        )

        with pytest.raises(FailedToReplaceS3BaseUrlWithProxyBaseUrlError):
            await S3Service(s3_client=s3_client_mock).create_file_url(
                s3_path=f"{faker.pystr()}/{faker.pystr()}",
                display_file_name=faker.pystr(),
                expires_in=faker.time_delta(),
                proxy_base_url=faker.url(),
            )



import dataclasses


@dataclasses.dataclass
class BaseError(Exception):
    def __str__(self) -> str:
        return self.__repr__().replace(self.__class__.__name__, "")


@dataclasses.dataclass
class KasperskyScanEngineThreatDetectedError(BaseError):
    response: bytes
    file_name: str


@dataclasses.dataclass
class NotAllowedMimeTypeError(BaseError):
    file_name: str
    mime_type: str
    allowed_mime_types: list[str]


@dataclasses.dataclass
class TooLargeFileError(BaseError):
    file_name: str
    file_size: int
    mime_type: str
    max_size: int


@dataclasses.dataclass
class FailedToConvertImageError(BaseError):
    file_name: str
    mime_type: str


@dataclasses.dataclass
class InvalidS3PathError(BaseError):
    s3_path: str


@dataclasses.dataclass
class FailedToReplaceS3BaseUrlWithProxyBaseUrlError(BaseError):
    s3_file_presigned_url: str
    proxy_base_url: str
